﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Collections.ObjectModel;
using System.Windows.Threading;
using System.Collections;
using System.Diagnostics;
using System.ComponentModel;

namespace ChartPerformanceTest
{
    public class PageBase : PhoneApplicationPage
    {
        private ObservableCollection<ViewModel> itemsSource;
        private DispatcherTimer timer;
        private DateTime lastDate;
        private Random random = new Random();
        private Stopwatch watch = new Stopwatch();
        private long layoutUpdatedTime;

        public PageBase()
        {
            this.Loaded += this.PageBase_Loaded;
            this.Unloaded += this.PageBase_Unloaded;
            this.LayoutUpdated += this.PageBase_LayoutUpdated;

            this.timer = new DispatcherTimer();
            this.timer.Interval = TimeSpan.FromMilliseconds(30);
            this.timer.Tick += this.OnTimer;

            this.FillData();

            this.BeginWatch();
        }

        public IList ChartData
        {
            get
            {
                return this.itemsSource;
            }
        }

        void PageBase_Unloaded(object sender, RoutedEventArgs e)
        {
            this.timer.Stop();
        }

        void PageBase_Loaded(object sender, RoutedEventArgs e)
        {
            this.timer.Start();
        }

        void PageBase_LayoutUpdated(object sender, EventArgs e)
        {
            if (DesignerProperties.IsInDesignTool)
            {
                return;
            }

            this.layoutUpdatedTime = this.watch.ElapsedMilliseconds;
            this.LayoutUpdated -= this.PageBase_LayoutUpdated;

            CompositionTarget.Rendering += this.CompositionTarget_Rendering;
        }

        void CompositionTarget_Rendering(object sender, EventArgs e)
        {
            CompositionTarget.Rendering -= this.CompositionTarget_Rendering;
            this.Dispatcher.BeginInvoke(this.StopWatch);
        }

        private void FillData()
        {
            this.itemsSource = new ObservableCollection<ViewModel>();
            this.lastDate = new DateTime(2000, 1, 1, 12, 0, 0);

            // add 100 points initially
            for (int i = 0; i < 100; i++)
            {
                this.lastDate = this.lastDate.AddMilliseconds(500);
                this.itemsSource.Add(this.CreateBusinessObject());
            }
        }

        private void OnTimer(object sender, EventArgs e)
        {
            this.lastDate = this.lastDate.AddMilliseconds(500);

            this.itemsSource.RemoveAt(0);
            this.itemsSource.Add(this.CreateBusinessObject());
        }

        private ViewModel CreateBusinessObject()
        {
            ViewModel obj = new ViewModel();

            obj.Value = this.random.Next(15, 50);
            obj.Date = this.lastDate;

            return obj;
        }

        public void BeginWatch()
        {
            this.watch.Reset();
            this.watch.Start();
        }

        public void StopWatch()
        {
            this.watch.Stop();
            this.Dispatcher.BeginInvoke(() => MessageBox.Show("Loading time: " + this.layoutUpdatedTime + "\r\nRendering Time: " + this.watch.ElapsedMilliseconds));
        }        
    }
}
